@@ -39,7 +39,7 @@ def create_report(plots, clusters, top_poses, best_energies, output="simulation_
     pdf.set_font('Arial', 'B', size=12)
     pdf.cell(0, 10, "Top poses", align='C')
     pdf.set_font('Arial', size=10)
-    top_poses_ordered = np.array(top_poses)[np.argsort(best_energies)]
+    top_poses_ordered = np.array(top_poses)[np.argsort(best_energies)[:len(top_poses)]]
     for i, poses in enumerate(top_poses_ordered[0:20]):
         if i == 0:
             pdf.ln(10)
@@ -64,9 +64,3 @@ def create_report(plots, clusters, top_poses, best_energies, output="simulation_
     #Output report    
     pdf.output(output, 'F')
     return output
-
-if __name__ == "__main__":
-    plots = glob.glob("/work/NBD_Utilities/PELE/PELE_Softwares/pele_platform_devel/pele_platform/results/Plots/*")
-    clusters = glob.glob("/work/NBD_Utilities/PELE/PELE_Softwares/pele_platform_devel/pele_platform/results/clusters/*.png")
-    top_poses = glob.glob("/work/NBD_Utilities/PELE/PELE_Softwares/pele_platform_devel/pele_platform/results/BestStructs/*.pdb")
-    create_report(plots, clusters, top_poses)
